import { EventEmitter, Component, forwardRef, Input, Output, ViewChild, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { fromTextArea } from 'codemirror';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
/**
 * CodeMirror component
 * Usage :
 * <lt-codemirror [(ngModel)]="data" [config]="{...}"></lt-codemirror>
 */
class LtCodemirrorComponent {
    /**
     * Constructor
     */
    constructor() {
        this.change = new EventEmitter();
        this.focus = new EventEmitter();
        this.blur = new EventEmitter();
        this.cursorActivity = new EventEmitter();
        this.instance = null;
        this._value = '';
    }
    /**
     * @return {?}
     */
    get value() { return this._value; }
    /**
     * @param {?} v
     * @return {?}
     */
    set value(v) {
        if (v !== this._value) {
            this._value = v;
            this.onChange(v);
        }
    }
    /**
     * On component destroy
     * @return {?}
     */
    ngOnDestroy() {
    }
    /**
     * On component view init
     * @return {?}
     */
    ngAfterViewInit() {
        this.config = this.config || {};
        this.codemirrorInit(this.config, this.size);
    }
    /**
     * Initialize codemirror
     * @param {?} config
     * @param {?} size
     * @return {?}
     */
    codemirrorInit(config, size) {
        this.instance = fromTextArea(this.host.nativeElement, config);
        this.instance.setValue(this._value);
        this.instance.on('change', () => {
            this.updateValue(this.instance.getValue());
        });
        this.instance.on('focus', (instance, event) => {
            this.focus.emit({ instance, event });
        });
        this.instance.on('cursorActivity', (instance) => {
            this.cursorActivity.emit({ instance });
        });
        this.instance.on('blur', (instance, event) => {
            this.blur.emit({ instance, event });
        });
        if (this.size != undefined) {
            this.instance.setSize(this.size.w, this.size.h);
        }
    }
    /**
     * Value update process
     * @param {?} value
     * @return {?}
     */
    updateValue(value) {
        this.value = value;
        this.onTouched();
        this.change.emit(value);
    }
    /**
     * Implements ControlValueAccessor
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        this._value = value || '';
        if (this.instance) {
            this.instance.setValue(this._value);
        }
    }
    /**
     * @param {?} _
     * @return {?}
     */
    onChange(_) { }
    /**
     * @return {?}
     */
    onTouched() { }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) { this.onChange = fn; }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) { this.onTouched = fn; }
}
LtCodemirrorComponent.decorators = [
    { type: Component, args: [{
                selector: 'lt-codemirror',
                providers: [
                    {
                        provide: NG_VALUE_ACCESSOR,
                        useExisting: forwardRef(() => LtCodemirrorComponent),
                        multi: true
                    }
                ],
                template: `
   <textarea #host></textarea>
  `
            }] }
];
/** @nocollapse */
LtCodemirrorComponent.ctorParameters = () => [];
LtCodemirrorComponent.propDecorators = {
    config: [{ type: Input }],
    size: [{ type: Input }],
    change: [{ type: Output }],
    focus: [{ type: Output }],
    blur: [{ type: Output }],
    cursorActivity: [{ type: Output }],
    host: [{ type: ViewChild, args: ['host',] }],
    instance: [{ type: Output }],
    value: [{ type: Input }]
};
if (false) {
    /** @type {?} */
    LtCodemirrorComponent.prototype.config;
    /** @type {?} */
    LtCodemirrorComponent.prototype.size;
    /** @type {?} */
    LtCodemirrorComponent.prototype.change;
    /** @type {?} */
    LtCodemirrorComponent.prototype.focus;
    /** @type {?} */
    LtCodemirrorComponent.prototype.blur;
    /** @type {?} */
    LtCodemirrorComponent.prototype.cursorActivity;
    /** @type {?} */
    LtCodemirrorComponent.prototype.host;
    /** @type {?} */
    LtCodemirrorComponent.prototype.instance;
    /** @type {?} */
    LtCodemirrorComponent.prototype._value;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
class LtCodemirrorModule {
}
LtCodemirrorModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule
                ],
                declarations: [LtCodemirrorComponent],
                exports: [LtCodemirrorComponent]
            },] }
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */

export { LtCodemirrorComponent, LtCodemirrorModule };
//# sourceMappingURL=lt-codemirror.js.map
