import { EventEmitter, Component, forwardRef, Input, Output, ViewChild, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { fromTextArea } from 'codemirror';
import { CommonModule } from '@angular/common';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
/**
 * CodeMirror component
 * Usage :
 * <lt-codemirror [(ngModel)]="data" [config]="{...}"></lt-codemirror>
 */
var LtCodemirrorComponent = /** @class */ (function () {
    /**
     * Constructor
     */
    function LtCodemirrorComponent() {
        this.change = new EventEmitter();
        this.focus = new EventEmitter();
        this.blur = new EventEmitter();
        this.cursorActivity = new EventEmitter();
        this.instance = null;
        this._value = '';
    }
    Object.defineProperty(LtCodemirrorComponent.prototype, "value", {
        get: /**
         * @return {?}
         */
        function () { return this._value; },
        set: /**
         * @param {?} v
         * @return {?}
         */
        function (v) {
            if (v !== this._value) {
                this._value = v;
                this.onChange(v);
            }
        },
        enumerable: true,
        configurable: true
    });
    /**
     * On component destroy
     */
    /**
     * On component destroy
     * @return {?}
     */
    LtCodemirrorComponent.prototype.ngOnDestroy = /**
     * On component destroy
     * @return {?}
     */
    function () {
    };
    /**
     * On component view init
     */
    /**
     * On component view init
     * @return {?}
     */
    LtCodemirrorComponent.prototype.ngAfterViewInit = /**
     * On component view init
     * @return {?}
     */
    function () {
        this.config = this.config || {};
        this.codemirrorInit(this.config, this.size);
    };
    /**
     * Initialize codemirror
     */
    /**
     * Initialize codemirror
     * @param {?} config
     * @param {?} size
     * @return {?}
     */
    LtCodemirrorComponent.prototype.codemirrorInit = /**
     * Initialize codemirror
     * @param {?} config
     * @param {?} size
     * @return {?}
     */
    function (config, size) {
        var _this = this;
        this.instance = fromTextArea(this.host.nativeElement, config);
        this.instance.setValue(this._value);
        this.instance.on('change', function () {
            _this.updateValue(_this.instance.getValue());
        });
        this.instance.on('focus', function (instance, event) {
            _this.focus.emit({ instance: instance, event: event });
        });
        this.instance.on('cursorActivity', function (instance) {
            _this.cursorActivity.emit({ instance: instance });
        });
        this.instance.on('blur', function (instance, event) {
            _this.blur.emit({ instance: instance, event: event });
        });
        if (this.size != undefined) {
            this.instance.setSize(this.size.w, this.size.h);
        }
    };
    /**
     * Value update process
     */
    /**
     * Value update process
     * @param {?} value
     * @return {?}
     */
    LtCodemirrorComponent.prototype.updateValue = /**
     * Value update process
     * @param {?} value
     * @return {?}
     */
    function (value) {
        this.value = value;
        this.onTouched();
        this.change.emit(value);
    };
    /**
     * Implements ControlValueAccessor
     */
    /**
     * Implements ControlValueAccessor
     * @param {?} value
     * @return {?}
     */
    LtCodemirrorComponent.prototype.writeValue = /**
     * Implements ControlValueAccessor
     * @param {?} value
     * @return {?}
     */
    function (value) {
        this._value = value || '';
        if (this.instance) {
            this.instance.setValue(this._value);
        }
    };
    /**
     * @param {?} _
     * @return {?}
     */
    LtCodemirrorComponent.prototype.onChange = /**
     * @param {?} _
     * @return {?}
     */
    function (_) { };
    /**
     * @return {?}
     */
    LtCodemirrorComponent.prototype.onTouched = /**
     * @return {?}
     */
    function () { };
    /**
     * @param {?} fn
     * @return {?}
     */
    LtCodemirrorComponent.prototype.registerOnChange = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) { this.onChange = fn; };
    /**
     * @param {?} fn
     * @return {?}
     */
    LtCodemirrorComponent.prototype.registerOnTouched = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) { this.onTouched = fn; };
    LtCodemirrorComponent.decorators = [
        { type: Component, args: [{
                    selector: 'lt-codemirror',
                    providers: [
                        {
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(function () { return LtCodemirrorComponent; }),
                            multi: true
                        }
                    ],
                    template: "\n   <textarea #host></textarea>\n  "
                }] }
    ];
    /** @nocollapse */
    LtCodemirrorComponent.ctorParameters = function () { return []; };
    LtCodemirrorComponent.propDecorators = {
        config: [{ type: Input }],
        size: [{ type: Input }],
        change: [{ type: Output }],
        focus: [{ type: Output }],
        blur: [{ type: Output }],
        cursorActivity: [{ type: Output }],
        host: [{ type: ViewChild, args: ['host',] }],
        instance: [{ type: Output }],
        value: [{ type: Input }]
    };
    return LtCodemirrorComponent;
}());
if (false) {
    /** @type {?} */
    LtCodemirrorComponent.prototype.config;
    /** @type {?} */
    LtCodemirrorComponent.prototype.size;
    /** @type {?} */
    LtCodemirrorComponent.prototype.change;
    /** @type {?} */
    LtCodemirrorComponent.prototype.focus;
    /** @type {?} */
    LtCodemirrorComponent.prototype.blur;
    /** @type {?} */
    LtCodemirrorComponent.prototype.cursorActivity;
    /** @type {?} */
    LtCodemirrorComponent.prototype.host;
    /** @type {?} */
    LtCodemirrorComponent.prototype.instance;
    /** @type {?} */
    LtCodemirrorComponent.prototype._value;
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */
var LtCodemirrorModule = /** @class */ (function () {
    function LtCodemirrorModule() {
    }
    LtCodemirrorModule.decorators = [
        { type: NgModule, args: [{
                    imports: [
                        CommonModule
                    ],
                    declarations: [LtCodemirrorComponent],
                    exports: [LtCodemirrorComponent]
                },] }
    ];
    return LtCodemirrorModule;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
 */

export { LtCodemirrorComponent, LtCodemirrorModule };
//# sourceMappingURL=lt-codemirror.js.map
